@@ -17,13 +17,12 @@ try:
     from urllib import urlencode
 except ImportError:
     from urllib.parse import urlencode
-from owslib.util import openURL, testXMLValue, extract_xml_list, ServiceException, xmltag_split
+from owslib.util import testXMLValue, extract_xml_list, ServiceException, xmltag_split, Authentication, openURL, log
 from owslib.etree import etree
 from owslib.fgdc import Metadata
 from owslib.iso import MD_Metadata
 from owslib.crs import Crs
 from owslib.namespaces import Namespaces
-from owslib.util import log
 from owslib.feature.schema import get_schema
 from owslib.feature.common import WFSCapabilitiesReader, \
     AbstractContentMetadata
@@ -62,7 +61,7 @@ class WebFeatureService_1_0_0(object):
     Implements IWebFeatureService.
     """
     def __new__(self,url, version, xml, parse_remote_metadata=False, timeout=30,
-                username=None, password=None):
+                username=None, password=None, auth=None):
         """ overridden __new__ method
 
         @type url: string
@@ -74,11 +73,12 @@ class WebFeatureService_1_0_0(object):
         @param timeout: time (in seconds) after which requests should timeout
         @param username: service authentication username
         @param password: service authentication password
+        @param auth: instance of owslib.util.Authentication
         @return: initialized WebFeatureService_1_0_0 object
         """
         obj=object.__new__(self)
         obj.__init__(url, version, xml, parse_remote_metadata, timeout,
-                     username=username, password=password)
+                     username=username, password=password, auth=auth)
         return obj
 
     def __getitem__(self,name):
@@ -88,17 +88,20 @@ class WebFeatureService_1_0_0(object):
         else:
             raise KeyError("No content named %s" % name)
 
-
     def __init__(self, url, version, xml=None, parse_remote_metadata=False, timeout=30,
-                 username=None, password=None):
+                 username=None, password=None, auth=None):
         """Initialize."""
+        if auth:
+            if username:
+                auth.username = username
+            if password:
+                auth.password = password
         self.url = url
         self.version = version
         self.timeout = timeout
-        self.username = username
-        self.password = password
+        self.auth = auth or Authentication(username, password)
         self._capabilities = None
-        reader = WFSCapabilitiesReader(self.version, self.username, self.password)
+        reader = WFSCapabilitiesReader(self.version, auth=self.auth)
         if xml:
             self._capabilities = reader.readString(xml)
         else:
@@ -130,7 +133,8 @@ class WebFeatureService_1_0_0(object):
         featuretypelist=self._capabilities.find(nspath('FeatureTypeList'))
         features = self._capabilities.findall(nspath('FeatureTypeList/FeatureType'))
         for feature in features:
-            cm=ContentMetadata(feature, featuretypelist, parse_remote_metadata)
+            cm = ContentMetadata(
+                feature, featuretypelist, parse_remote_metadata, auth=self.auth)
             self.contents[cm.id]=cm
 
         #exceptions
@@ -141,9 +145,9 @@ class WebFeatureService_1_0_0(object):
         """Request and return capabilities document from the WFS as a
         file-like object.
         NOTE: this is effectively redundant now"""
-        reader = WFSCapabilitiesReader(self.version)
-        return openURL(reader.capabilities_url(self.url), timeout=self.timeout,
-                       username=self.username, password=self.password)
+        reader = WFSCapabilitiesReader(self.version, auth=self.auth)
+        return openURL(reader.capabilities_url(self.url),
+                       timeout=self.timeout, auth=self.auth)
 
     def items(self):
         '''supports dict-like items() access'''
@@ -235,9 +239,7 @@ class WebFeatureService_1_0_0(object):
 
         data = urlencode(request)
         log.debug("Making request: %s?%s" % (base_url, data))
-        u = openURL(base_url, data, method, timeout=self.timeout,
-                    username=self.username, password=self.password)
-
+        u = openURL(base_url, data, method, timeout=self.timeout, auth=self.auth)
 
         # check for service exceptions, rewrap, and return
         # We're going to assume that anything with a content-length > 32k
@@ -283,8 +285,7 @@ class WebFeatureService_1_0_0(object):
         Get layer schema compatible with :class:`fiona` schema object
         """
 
-        return get_schema(self.url, typename, self.version)
-
+        return get_schema(self.url, typename, self.version, auth=self.auth)
 
 
 class ServiceIdentification(object):
@@ -314,8 +315,9 @@ class ContentMetadata(AbstractContentMetadata):
     Implements IMetadata.
     """
 
-    def __init__(self, elem, parent, parse_remote_metadata=False, timeout=30):
+    def __init__(self, elem, parent, parse_remote_metadata=False, timeout=30, auth=None):
         """."""
+        super(ContentMetadata, self).__init__(auth)
         self.id = testXMLValue(elem.find(nspath('Name')))
         self.title = testXMLValue(elem.find(nspath('Title')))
         self.abstract = testXMLValue(elem.find(nspath('Abstract')))
@@ -378,7 +380,8 @@ class ContentMetadata(AbstractContentMetadata):
             if metadataUrl['url'] is not None \
                     and metadataUrl['format'].lower() == 'xml':
                 try:
-                    content = openURL(metadataUrl['url'], timeout=timeout)
+                    content = openURL(
+                        metadataUrl['url'], timeout=timeout, auth=self.auth)
                     doc = etree.fromstring(content.read())
                     if metadataUrl['type'] == 'FGDC':
                         mdelem = doc.find('.//metadata')
