@@ -30,7 +30,7 @@ import six
 from owslib.etree import etree
 from owslib.util import (openURL, testXMLValue, extract_xml_list,
                          xmltag_split, OrderedDict, ServiceException,
-                         bind_url, nspath_eval)
+                         bind_url, nspath_eval, Authentication)
 from owslib.fgdc import Metadata
 from owslib.iso import MD_Metadata
 from owslib.map.common import WMSCapabilitiesReader, AbstractContentMetadata
@@ -57,25 +57,24 @@ class WebMapService_1_1_1(object):
         else:
             raise KeyError("No content named %s" % name)
 
-    def __init__(self, url, version='1.1.1', xml=None,
-                 username=None,
-                 password=None,
-                 parse_remote_metadata=False,
-                 headers=None,
-                 timeout=30):
+    def __init__(self, url, version='1.1.1', xml=None, username=None, password=None,
+                 parse_remote_metadata=False, headers=None, timeout=30, auth=None):
         """Initialize."""
+        if auth:
+            if username:
+                auth.username = username
+            if password:
+                auth.password = password
         self.url = url
-        self.username = username
-        self.password = password
         self.version = version
         self.timeout = timeout
         self.headers = headers
         self._capabilities = None
+        self.auth = auth or Authentication(username, password)
 
         # Authentication handled by Reader
-        reader = WMSCapabilitiesReader(self.version, url=self.url,
-                                       un=self.username, pw=self.password,
-                                       headers=headers)
+        reader = WMSCapabilitiesReader(
+            self.version, url=self.url, headers=headers, auth=self.auth)
         if xml:  # read from stored xml
             self._capabilities = reader.readString(xml)
         else:  # read from server
@@ -149,8 +148,7 @@ class WebMapService_1_1_1(object):
         NOTE: this is effectively redundant now"""
 
         reader = WMSCapabilitiesReader(
-            self.version, url=self.url, un=self.username, pw=self.password
-        )
+            self.version, url=self.url, auth=self.auth)
         u = self._open(reader.capabilities_url(self.url))
         # check for service exceptions, and return
         if u.info()['Content-Type'] == 'application/vnd.ogc.se_xml':
@@ -266,10 +264,10 @@ class WebMapService_1_1_1(object):
 
         self.request = bind_url(base_url) + data
 
-        u = openURL(base_url, data, method, username=self.username, password=self.password, timeout=timeout or self.timeout)
+        u = openURL(base_url, data, method, timeout=timeout or self.timeout, auth=self.auth)
 
         # check for service exceptions, and return
-        if u.info()['Content-Type'].split(';')[0] in ['application/vnd.ogc.se_xml']:
+        if u.info().get('Content-Type', '').split(';')[0] in ['application/vnd.ogc.se_xml']:
             se_xml = u.read()
             se_tree = etree.fromstring(se_xml)
             err_message = six.text_type(se_tree.find('ServiceException').text).strip()
@@ -332,7 +330,7 @@ class WebMapService_1_1_1(object):
 
         self.request = bind_url(base_url) + data
 
-        u = openURL(base_url, data, method, username=self.username, password=self.password, timeout=timeout or self.timeout)
+        u = openURL(base_url, data, method, timeout=timeout or self.timeout, auth=self.auth)
 
         # check for service exceptions, and return
         if u.info()['Content-Type'] == 'application/vnd.ogc.se_xml':
@@ -377,7 +375,10 @@ class ServiceProvider(object):
             self.name=name.text
         else:
             self.name=None
-        self.url=self._root.find('OnlineResource').attrib.get('{http://www.w3.org/1999/xlink}href', '')
+        self.url = None
+        online_resource = self._root.find('OnlineResource')
+        if online_resource is not None:
+            self.url = online_resource.attrib.get('{http://www.w3.org/1999/xlink}href', '')
         #contact metadata
         contact = self._root.find('ContactInformation')
         ## sometimes there is a contact block that is empty, so make
@@ -407,7 +408,10 @@ class ContentMetadata(AbstractContentMetadata):
 
     Implements IContentMetadata.
     """
-    def __init__(self, elem, parent=None, children=None, index=0, parse_remote_metadata=False, timeout=30):
+
+    def __init__(self, elem, parent=None, children=None, index=0,
+                 parse_remote_metadata=False, timeout=30, auth=None):
+        super(ContentMetadata, self).__init__(auth)
         if elem.tag != 'Layer':
             raise ValueError('%s should be a Layer' % (elem,))
 
@@ -590,7 +594,8 @@ class ContentMetadata(AbstractContentMetadata):
             if metadataUrl['url'] is not None \
                     and metadataUrl['format'].lower() in ['application/xml', 'text/xml']:  # download URL
                 try:
-                    content = openURL(metadataUrl['url'], timeout=timeout)
+                    content = openURL(
+                        metadataUrl['url'], timeout=timeout, auth=self.auth)
                     doc = etree.fromstring(content.read())
 
                     if metadataUrl['type'] == 'FGDC':
